;  ***************************************************************************
;  *                                                                         *
;  * File:   getdate.asm          Version  1.10            Date: 2009-02-26  *
;  *                                                                         *
;  * Copyright (C) 1991-2009 by Kostis Netzwerkberatung                      *
;  * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056                     *
;  * kosta@kostis.net (Konstantinos Kostis), http://www.kostis.net/          *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Function:                                                               *
;  *     - return DOS date & time info in DOS errorlevel                     *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Parameters:                                                             *
;  *                                                                         *
;  *    /S      seconds (0..59)         second                               *
;  *    /T      time    (0..59)         minute                               *
;  *    /H      hour    (0..23)	        hour                                 *
;  *    /U      hhm     (0..235)        10 * hour + minute / 10              *
;  *    /W      weekday (1..7)          Sunday..Saturday                     *
;  *    /D      day     (1..31)	        day                                  *
;  *    /M      month   (1..12)	        January..December                    *
;  *    /Y      year    (80..138)       1900 + x                             *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * Comments:                                                               *
;  *                                                                         *
;  * With Microsoft's Macro Assembler 6.0 type:   ml /AT /cp getdate.asm     *
;  * With Borland's   Turbo Assembler 2.5 type:   tasm /ml getdate.asm       *
;  *                                              tlink /t getdate.obj       *
;  *                                                                         *
;  ***************************************************************************
;  *                                                                         *
;  * History:                                                                *
;  *    2009-02-26 KK V1.10                                                  *
;  *        - updated information                                            *
;  *    1998-11-14 KK V1.01                                                  *
;  *        - used macros                                                    *
;  *        - changed /H to /U                                               *
;  *        - added new /H giving just the hour                              *
;  *        - now explaining switches when displaying syntax                 *
;  *    1991-11-09 KK V1.00                                                  *
;  *        - initial coding                                                 *
;  *                                                                         *
;  ***************************************************************************

.MODEL	TINY

.CONST

;  ***************************************************************************
;
;	Program Constants
;
;  ***************************************************************************

TAB		EQU	9
CR		EQU	13
LF		EQU	10
EOS		EQU	'$'
FLAGLEN		EQU	3
SYNTAXERR	EQU	255

;  ***************************************************************************
;
;	MS-DOS service functions
;
;  ***************************************************************************

INT_OLDTERM	EQU	20h		; terminate DOS 1.x program
INT_DOS		EQU	21h		; MS-DOS Service Interrupt
   DOS_PUTSTR	EQU	09h		;    display string terminated by '$'
   DOS_GETDATE	EQU	2Ah		;    get date
   DOS_GETTIME	EQU	2Ch		;    get time
   DOS_GETVER	EQU	30h		;    return DOS version in AL:AH
   DOS_TERM	EQU	4Ch		;    terminate program

;  ***************************************************************************
;
;	personal C style macros to ease coding
;
;  ***************************************************************************

doscall		MACRO	Function		; call MS-DOS Function

		mov	ah,Function		;
		int	INT_DOS			;

		ENDM				;

puts		MACRO	String			; write string to stdout

		lea	dx,String		; must be terminated by a '$'
		doscall	DOS_PUTSTR		;

		ENDM				;

;  ***************************************************************************
;
;	Pointers to command line parameters
;
;  ***************************************************************************

ARGC		EQU	80h	; argc (length of command line parameters)
ARGV		EQU	82h	; command line parameters (excl. lead. blank)

.DATA

;  ***************************************************************************
;
;	Program Messages
;
;  ***************************************************************************
							  
hello	DB	'Copyright (C) 1991-2009 by Kostis Netzwerkberatung', CR, LF
	DB	'Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056', CR, LF
	DB	'kosta@kostis.net (Konstantinos Kostis), http://www.kostis.net/', CR, LF
	DB	CR, LF
	DB	'getdate V1.10 (2009-02-26)', CR, LF
	DB	CR, LF, EOS

syntax	DB	'Syntax: getdate /S | /T | /H | /U | /W | /D | /M | /Y', CR, LF
	DB	CR, LF
	DB	TAB, '/S     returns the second                        (1..60)', CR, LF
	DB	TAB, '/T     returns the minute                        (1..60)', CR, LF
	DB	TAB, '/H     returns the hour                          (0..23)', CR, LF
	DB	TAB, '/U     returns 10 * hour + minute / 10           (0..235)', CR, LF
	DB	TAB, '/W     returns the weekday (1 = sunday)          (1..7)', CR, LF
	DB	TAB, '/D     returns the day of the month              (1..31)', CR, LF
	DB	TAB, '/M     returns the month                         (1..12)', CR, LF
	DB	TAB, '/Y     returns the year (add 1900 for the year)  (80..138)', CR, LF
	DB	CR, LF, EOS

flags	DB	'STHUWDMY', 0

;  ***************************************************************************
;
;	Program Data and flags must have exact same order
;
;  ***************************************************************************

second	DB	0
minute	DB	0
hour	DB	0
hhm	DB	0
weekday	DB	0
day	DB	0
month	DB	0
year	DB	0

;  ***************************************************************************
;
;	Main Program
;
;  ***************************************************************************

.CODE

		ORG	100h		; DOS-COM prog.

;  ***************************************************************************
;	get date
;  ***************************************************************************
							  
start:		doscall	DOS_GETDATE	; get date from DOS

		sub	cx, 1900	; correct year to be 80..255
		mov	year, cl	;
		mov	month, dh	;
		mov	day, dl		;
		inc	al		;
		mov	weekday, al	; correct weekday to be 1..7

;  ***************************************************************************
;	get time
;  ***************************************************************************

		doscall	DOS_GETTIME	; get time from DOS

		mov	second, dh	;
		mov	minute, cl	;
		mov	hour,ch		;

		mov	al, 10		; calculate hhm (0..235)
		mul	ch		; ax = 10 * hour
		mov	bl, al		; bl = 10 * hour (ignore ah)

		mov	al, cl		; ax = minute (assuming ah = 0)
		cbw			;
		mov	cl, 10		;
		div	cl		; al = minute / 10 (al = ax / cl)

		add	bl, al		; bl = 10 * hour + minute / 10 ;
		mov	hhm, bl		;

;  ***************************************************************************
;	check parameters
;  ***************************************************************************

		mov	si, ARGC	; COMMAND-Line parameter check for flag
		mov	cl, byte ptr[si] ;
		cmp 	cl, FLAGLEN	; parameter length must be FLAGLEN
		jne	synerr		; only one parameter supported -> '/?'

		mov	si, ARGV	; get first parameter byte
		mov	cl, byte ptr[si] ;
		cmp	cl, '/'		; '/' and '-' supported
		je	flag		;
		cmp	cl, '-'		;
		jne	synerr		;

flag:		inc	si		; get flag
		mov	cl, byte ptr[si] ;
		and	cl, 0DFh	; keep it to upper case
		mov	di, 0		;

;  ***************************************************************************
;	loop thru option table
;  ***************************************************************************

search:		mov	al, flags [di]	; load flag from table
		or	al, al		; check for end of table
		jz	synerr		; end of flag table -> syntax error

		cmp	cl, al		; compare table with flag
		je	gotflag		; if flag is found, return its value

		inc	di		; else look at the next table entry
		jmp	search		; continue loop

;  ***************************************************************************
;	terminate program returning data in return code
;  ***************************************************************************

gotflag:	mov	al, second [di]	; get the right parameter
exit:		doscall	DOS_TERM	; return to DOS

;  ***************************************************************************
;	give syntax (error message)
;  ***************************************************************************

synerr:		puts	hello		; display copyright info
		puts	syntax		; display syntax
		mov	al, SYNTAXERR	; return code for syntax error
		jmp	exit		; and exit program

		END	start		;

.MODEL          TINY
